/*------------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        20.03.2014
 Description:    DMX_4 Kanal_Controller


 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
------------------------------------------------------------------------------*/

#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/io.h>
#include <stdio.h>

#define F_CPU 12000000

#define LED_R_PIN_OUT DDRD |= (1<<PD5);
#define LED_G_PIN_OUT DDRD |= (1<<PD4);
#define LED_B_PIN_OUT DDRD |= (1<<PD3);

#define LED_R_ON		PORTD |= (1<<PD4);
#define LED_R_OFF		PORTD &= ~(1<<PD4);
#define LED_B_ON		PORTD |= (1<<PD5);
#define LED_B_OFF		PORTD &= ~(1<<PD5);
#define LED_G_ON		PORTD |= (1<<PD3);
#define LED_G_OFF		PORTD &= ~(1<<PD3);


#define DMX_BAUD 250000
#define DMX_LOST_TIMEOUT 8000

volatile unsigned int dmx_lost = DMX_LOST_TIMEOUT;
volatile unsigned int pwm_out[3];

volatile unsigned char usart_get = 0;
volatile unsigned char usart_data = 0;

const PROGMEM uint16_t pwmtable_10[255] =
{
	0,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,3,3,3,3,3,3,3,3,3,3,3,3,3,4,4,4,4,4,
	4,4,4,4,5,5,5,5,5,5,5,6,6,6,6,6,6,7,7,7,7,7,7,8,8,8,8,9,9,9,9,10,10,10,10,11,11,11,12,12,12,13,13,13,14,14,14,
	15,15,16,16,16,17,17,18,18,19,19,20,20,21,22,22,23,23,24,25,25,26,27,28,28,29,30,31,32,32,33,34,35,36,37,38,39,
	40,41,43,44,45,46,47,49,50,51,53,54,56,57,59,61,62,64,66,68,69,71,73,75,77,79,82,84,86,89,91,94,96,99,102,104,
	107,110,113,116,120,123,126,130,133,137,141,145,149,153,157,161,166,170,175,180,185,190,195,200,206,211,217,223,
	229,236,242,249,256,263,270,278,285,293,301,309,318,327,336,345,354,364,374,385,395,406,417,429,441,453,465,478,
	491,505,519,533,548,563,578,594,610,627,644,662,680,699,719,738,759,780,801,823,846,869,893,918,943,969,996,1023
};

#define PWM_TO_CIEL(x) (pgm_read_word(&pwmtable_10[x]))

//############################################################################
//Hier wird die Zeit gezhlt (Tick 1ms)
ISR (TIMER0_COMPA_vect)
//############################################################################
{
	static unsigned int pwm_counter = 0;

	pwm_counter++;
	
	if(pwm_counter == 1024)
	{
		pwm_counter = 0;
		if(pwm_out[0]!=0) LED_R_ON;
		if(pwm_out[1]!=0) LED_G_ON;
		if(pwm_out[2]!=0) LED_B_ON;
	}
	
	if(pwm_counter > pwm_out[0]) 
	{
		LED_R_OFF;
	}

	if(pwm_counter > pwm_out[1])
	{
		LED_G_OFF;
	}

	if(pwm_counter > pwm_out[2]) 
	{
		LED_B_OFF;
	}
}

//############################################################################
//Hauptprogramm
int main (void) 
//############################################################################
{  
	static unsigned int dmx_channel_rx_count = 0;
	static unsigned char dmx_valid = 0;
	
	unsigned int dmx_adresse = 0;
	unsigned char dmx_buffer[4] = {0,0,0,0};
	unsigned int dmx_adresse_tmp;
	unsigned char tmp = 0;

	//Init usart DMX-BUS
	UBRRH   = (unsigned char)(F_CPU / (DMX_BAUD * 16L) - 1)>>8;
	UBRRL   = (unsigned char)(F_CPU / (DMX_BAUD * 16L) - 1);
	
	UCSRB|=(1 << RXEN); //| 1<< RXCIE
	UCSRC|=(1<<USBS); //USBS0 2 Stop bits	
	
	//Timer0 Timecounter fr DMX Ausfall und PWM
	TCCR0A |= (1<<WGM01);
	TCCR0B |= (1<<CS00);
	TIMSK |= (1<<OCIE0A);
	OCR0A = 150; 
	
	PORTB |= 0xFF;
	PORTD |= (1<<PD6);
	
	DDRD |=(1<<PD2);
	PORTD &=~(1<<PD2);
	
	LED_R_PIN_OUT;
	LED_G_PIN_OUT;
	LED_B_PIN_OUT;
	
	LED_R_ON;
	for(unsigned long tmp = 0;tmp<500000;tmp++)asm("nop");
	LED_R_OFF;
	LED_G_ON;
	for(unsigned long tmp = 0;tmp<500000;tmp++)asm("nop");
	LED_G_OFF;
	LED_B_ON;
	for(unsigned long tmp = 0;tmp<500000;tmp++)asm("nop");
	LED_B_OFF;
	
	sei();//Globale Interrupts Enable
	
	//Endlosschleife
	while(1)
	{
		dmx_adresse_tmp = (PINB ^ 0xFF)<<1;
		if(!(PIND&(1<<PD6))) dmx_adresse_tmp |= 0x01;
		
		if(dmx_adresse != dmx_adresse_tmp) dmx_adresse =  dmx_adresse_tmp;
		
		pwm_out[0] = PWM_TO_CIEL(dmx_buffer[1]);
		pwm_out[1] = PWM_TO_CIEL(dmx_buffer[2]);
		pwm_out[2] = PWM_TO_CIEL(dmx_buffer[3]);	
		
		if(UCSRA&(1<<RXC))
		{
			tmp =  UDR;
			if(UCSRA&(1<<FE)){
				dmx_channel_rx_count = 0;	
				dmx_buffer[0] = tmp;
				if(dmx_buffer[0] == 0)
				{
					dmx_valid = 1;
					dmx_channel_rx_count++;
				}
				else
				{
					dmx_valid = 0;
				}
			}else{
				if(dmx_valid)
				{
					if(dmx_channel_rx_count == dmx_adresse) dmx_buffer[1] = tmp;
					if(dmx_channel_rx_count == dmx_adresse+1) dmx_buffer[2] = tmp;
					if(dmx_channel_rx_count == dmx_adresse+2) dmx_buffer[3] = tmp;
				
					if(dmx_channel_rx_count < 514)
					{
						dmx_channel_rx_count++;
					}
				}
			}
			UCSRA = 0;
		}
	
	
	
	
	
	}
}

